@@ -25,6 +25,7 @@
 #include <errno.h>
 #include <dirent.h>
 #include <stdbool.h>
+#include "kernel-shared/uapi/btrfs.h"
 #include "kernel-shared/ctree.h"
 #include "kernel-shared/volumes.h"
 #include "common/open-utils.h"
@@ -33,7 +34,6 @@
 #include "common/messages.h"
 #include "common/help.h"
 #include "cmds/commands.h"
-#include "ioctl.h"
 
 static const char * const balance_cmd_group_usage[] = {
 	"btrfs balance <command> [options] <path>",
@@ -366,7 +366,7 @@ static const char * const cmd_balance_start_usage[] = {
 	"a delay to stop it.",
 	"",
 	OPTLINE("-d[filters]", "act on data chunks with optional filters (no space in between)"),
-	OPTLINE("-m[filters]", "act on metadata chunks with optinal filters (no space in between)"),
+	OPTLINE("-m[filters]", "act on metadata chunks with optional filters (no space in between)"),
 	OPTLINE("-s[filters]", "act on system chunks (only under -f) with optional filters (no space in between)"),
 	OPTLINE("-f", "force a reduction of metadata integrity, or skip timeout when converting to RAID56 profiles"),
 	OPTLINE("--full-balance", "do not print warning and do not delay start"),
